import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Random;
import java.util.Scanner;


public class FMeasure extends Evaluation{

	private boolean matrix [][];
	private int row, col;
	
	private boolean target [];
	private int target_word_num = 0;
	private double lambda;
	
	private int[] existing_word;
	
	public int get_n() {
		return row;
	}

	
	public FMeasure(String data_file_name, String target_file_name, double lambda) throws IOException{
		this.lambda = lambda;
//		System.out.println("loading ...");
//		System.out.println(data_file_name);
		File file = new File(data_file_name);
		Scanner scan = new Scanner(file);
		String str [] = scan.nextLine().trim().split("\\s+");
		col = str.length;
		row = 1;
		while (scan.hasNextLine()){
			scan.nextLine();
			row ++;
		}
		scan.close();
		scan = null;
		scan = new Scanner(file);
		matrix = new boolean [row][col];
		for (int i = 0; i < row; i ++){
			for (int j = 0; j < col; j ++){
				matrix[i][j] = scan.nextDouble() == 0 ? false : true;
			}
		}
		scan.close();
		scan = null;
		file = null;
		
		//to make tao(x) faster, bacause we can skip some words if the matrix is sparse
		find_existing_word();
//		System.out.println("word number : " + col);
//		System.out.println("useful word : " + existing_word.length);
//		System.out.println(target_file_name);
		file = new File(target_file_name);
		scan = new Scanner(file);
		str = scan.nextLine().trim().split("\\s+");
		if (str.length != col){
			System.out.println(str);
			System.err.println("size err: \ntarget vector length : " + str.length + "\nword vector length : " + col);
			System.exit(0);
		}
		target = new boolean[col];
		for (int i = 0; i < target.length; i ++){
			target[i] = Integer.parseInt(str[i]) == 1 ? true : false;
		}
		target_word_num = words_number(target);
		scan.close();
	}
	
	public void find_existing_word(){
		if (col==0 || row==0)
			return;
		List<Integer> list = new ArrayList<Integer>();
		for (int j = 0; j < col; j ++){
			for (int i = 0; i < row; i ++){
				if (matrix[i][j]){
					list.add(j);
					break;
				}
			}
		}
		existing_word = new int[list.size()];
		for (int i = 0; i < list.size(); i ++){
			existing_word[i] = list.get(i);
		}
//		System.out.println("existing word num : " + existing_word.length);
		list = null;
	}
	
	public static void create_synthetic_data(int V, int W, double p, String syn_file_name){
		Random rand = new Random();
		int [][] matrix = new int [V][W];
		StringBuffer sb = new StringBuffer();
		for (int i = 0; i < V; i ++){
			for (int j = 0; j < W; j ++){
				matrix[i][j] = rand.nextDouble() < p ? 1 : 0;
				sb.append(matrix[i][j] + " ");
			}
			log_to_file(syn_file_name, sb.toString());
			sb.setLength(0);
		}
	}
	
	public static void create_synthetic_target(int W, int size, String tar_file_name){
		List<Integer> list = new ArrayList<Integer>();  
        for (int i = 0; i < W; i++)  
            list.add(new Integer(i));
        Collections.shuffle(list);
        int tar [] = new int [W];
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < size; i ++){
        	tar[list.get(i)] = 1;
        }
        for (int i = 0; i < tar.length; i ++){
        	sb.append(tar[i] + " ");
        }
        log_to_file(tar_file_name, sb.toString());
	}
	
	
	public boolean [] tao(boolean [] s){
		boolean [] words = new boolean [col];
		if (existing_word.length < col){ // 
			for (int i = 0; i < row; i ++){
				if (s[i]){
					for (int j = 0; j < existing_word.length; j ++){
						int index = existing_word[j];
						words [index] = words[index] || matrix[i][index];
					}
				}
			}
		}
		else{
			for (int i = 0; i < row; i ++){
				if (s[i]){
					for (int j = 0; j < col; j ++){
						words [j] = words[j] || matrix[i][j];
					}
				}
			}
		}
		
		return words;
	}
	
	
	
	public boolean [] intersection (boolean [] s1, boolean [] s2){
		if (s1.length != s2.length){
			System.out.println("intersection error");
		}
		boolean [] intersect = new boolean [s1.length];
		for (int i = 0; i < s1.length; i ++){
			intersect [i] = s1[i] && s2[i] ;
		}
		return intersect;
	}
	
	public int words_number(boolean [] words){
		int num = 0;
		for (int i = 0; i < words.length; i ++){
			if (words[i])
				num ++;
		}
		return num;
	}
	
	public double g(boolean [] s){
		return words_number( intersection(tao(s), target));
	}
	
	public double f(boolean [] s){
		return lambda * target_word_num + (1-lambda) * words_number(tao(s));
	}
	
	public static void  log_to_file(String filename, String log_content) {
		try{
			FileWriter fileWritter = new FileWriter(filename,true);
			BufferedWriter bufferWritter = new BufferedWriter(fileWritter);
			bufferWritter.write(log_content + "\n");
			bufferWritter.close();
		}
		catch(IOException e){
			e.printStackTrace();
			}
	}
	
	public void create_log_file(String filename) {
		try{
			File file =new File(filename);
			file.createNewFile();
			file = null;
		}catch(IOException e){
			e.printStackTrace();
		}
	}
}
