import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Random;



public class PORM {
	public Evaluation eval = null;
	public LinkedList<Solution> population = null;
	public int n;
	public int total_iter_num = 1000;
	public String output_file_name = "porm_output.txt";
	
	public PORM(Evaluation eval) {
		this.eval = eval;
		this.n = eval.get_n();
		
	}
	
	public Solution  run_porm(){
		Random rand = new Random();
		population = new LinkedList<Solution>();
		Solution initial_solution = random_solution(n);
		initial_solution.f_value = eval.f(initial_solution.get_x_ref());
		initial_solution.g_value = eval.g(initial_solution.get_x_ref());
		initial_solution.set_value(initial_solution.f_value / initial_solution.g_value);	
		population.add(initial_solution);
		long t = System.currentTimeMillis();
		log_to_file(output_file_name, " ");
		for (int i = 0; i < total_iter_num; i ++){
			int r = rand.nextInt(population.size());
			Solution s = population.get(r).mutation();
			s.f_value = eval.f(s.get_x_ref());
			s.g_value = eval.g(s.get_x_ref());
			s.set_value(s.g_value == 0 ? Double.MAX_VALUE : s.f_value / s.g_value);
			
			if (can_accepted(s)){
				Iterator <Solution> iter = population.iterator();
				while (iter.hasNext()){
					Solution z = iter.next();
					if (s.f_value <= z.f_value && s.g_value >= z.g_value){
						iter.remove();
					}
				}
				iter = null;
				//ignore this sentence because s will be removed in the future
				//population.add(s);
				LinkedList<Solution> q = new LinkedList<Solution>();
				Solution min_f = s;
				Solution max_g = s;
				Solution min_f_g = s;
				
				iter = population.iterator();
				while(iter.hasNext()){
					Solution z = iter.next();
					if (z.get_norm() == s.get_norm()){
						iter.remove();
						q.add(z);
						if (z.f_value < min_f.f_value)
							min_f = z;
						if (z.g_value > max_g.g_value)
							max_g = z;
						if (z.get_value() < min_f_g.get_value())
							min_f_g = z;
					}
				}
				iter = null;
				population.add(min_f_g);
				if (min_f_g != min_f){
					population.add(min_f);
					if (max_g != min_f && max_g != min_f_g){
						population.add(max_g);
					}
				}
				else if (max_g != min_f){
					population.add(max_g);
				}
			}
		}
		log_to_file(output_file_name, "result=" + population_info());
		log_to_file(output_file_name, "t=" + (System.currentTimeMillis() - t) + ";");
		return current_best();
	}
	
	public static void  log_to_file(String filename, String log_content) {
		try{
			FileWriter fileWritter = new FileWriter(filename,true);
			BufferedWriter bufferWritter = new BufferedWriter(fileWritter);
			bufferWritter.write(log_content + "\n");
			bufferWritter.close();
		}
		catch(IOException e){
			e.printStackTrace();
			}
	}
	
	public String population_info(){
		StringBuffer sb = new StringBuffer();
		sb.append("[\n");
		for (Solution z : population){
			sb.append(z.get_value() + "," + z.get_norm() + "," + z.f_value + "," + z.g_value +  "\n");
		}
		sb.append("];");
		return sb.toString();
	}
	
	public void set_iter_num(int num){
		this.total_iter_num = num;
	}
	
	public Solution current_best(){
		Solution best = population.get(0);
		for (Solution s : population){
			if (s.get_value() < best.get_value()){
				best = s;
			}
		}
		return best;
	}
	
	public boolean can_accepted(Solution s){
		boolean accept = true;
		for (Solution z : population){
			if ( (z.f_value < s.f_value && z.g_value >= s.g_value) || 
					(z.f_value <= s.f_value && z.g_value > s.g_value ) ){
				accept = false;
				break;
			}
		}
		return accept;
	}
	
	public Solution random_solution(int n){
		boolean [] b = new boolean[n];
		Random rand = new Random();
		for (int i = 0; i < n; i ++){
			b[i] = rand.nextDouble() < 0.5 ? true : false;
		}
		return new Solution(b);
	}

}
